@@ -1,27 +1,27 @@
 # maptree package
 #   for graphing and mapping of hierarchical clustering and
 #   regression trees
-# denis white, us epa, 6 June 2001, version 1.1-1
+# denis white, us epa, 15 March 2002, version 1.2-1
 #
 # function calls
 #
 ############################################################
 #
-# draw.clust <- function (cluster, ps=par("ps"), size=10, 
+# draw.clust <- function (cluster, cex=par("cex"), size=2, 
 #     col=NULL)
 #
-# draw.tree <- function (tree, ps=par("ps"), size=10, col=NULL, 
+# draw.tree <- function (tree, cex=par("cex"), size=2, col=NULL, 
 #     nodeinfo=FALSE, units="units", cases="obs", digits=0 )
 #
 # group.clust <- function (cluster, k=NULL, h=NULL)
 #
 # group.tree <- function (tree)
 #
-# map.groups <- function (pts, group, pch=19, size=5, col=NULL,
+# map.groups <- function (pts, group, pch=19, size=2, col=NULL,
 #     border=NULL)
 #
-# map.key <- function (x, y, lables=NULL, new=FALSE, size=5, 
-#     ps=par("ps"), pch=19, head="", sep=1, col=NULL)
+# map.key <- function (x, y, lables=NULL, new=FALSE, size=2, 
+#     cex=par("cex"), pch=19, head="", sep=0.2, col=NULL)
 #
 # ngon <- function (xydc, n=4, angle=0, type=1)
 #
@@ -31,33 +31,34 @@
 
 ############################################################
 
-draw.clust <- function (cluster, ps=par("ps"), size=10, 
+draw.clust <- function (cluster, cex=par("cex"), size=2, 
   col=NULL)
   # cluster is class hclust or twins
-  # ps is par parameter, pointsize of text
-  # size is in mm of colored square at observations
+  # cex is par parameter, size of text
+  # size is length of a side of a colored square in cex  
+  #   units drawn at the observations
   # if col is NULL, use rainbow(),
   #   else if col="gray" or "grey", use gray()
   # returned value is col or generated colors
 {
-  if (class(cluster) == "hclust") clust <- cluster
-  else if (inherits(cluster,"twins"))
+  if (class (cluster) == "hclust") clust <- cluster
+  else if (inherits (cluster, "twins")) 
     clust <- as.hclust (cluster)
-  else 
-    stop("draw: input not hclust or twins")
+  else stop("draw.clust: input not hclust or twins")
   merg <- clust$merge
-  nmerg <- nrow(merg)
-  if (nmerg<2) stop("draw: <3 clusters")
+  nmerg <- nrow (merg)
+  if (nmerg<2) stop ("draw: < 3 clusters")
   hite <- clust$height
-  cord <- order(clust$order)
+  cord <- order (clust$order)
   xmax <- nrow (merg) + 1
   ymax <- max (hite)
   pinx <- par ("pin")[1]
   piny <- par ("pin")[2]
   xmin <- 1
-  box <- size/25.4
+  chr <- par("cin")[2] * cex
+  box <- size * chr
   xscale <- (xmax-xmin)/pinx
-  xbh <- xscale*box/2
+  xbh <- xscale * box / 2
   tail <- 0.25
   yscale <- ymax/(piny - tail)
   ytail <- yscale*tail
@@ -70,11 +71,11 @@ draw.clust <- function (cluster, ps=par("ps"), size=10,
   y1 <- ymin - yf
   y2 <- ymax + yf
   plot (c(x1,x2),c(y1,y2),type="n",axes=FALSE,xlab="",ylab="")
-  oldps <- par ("ps")
-  par (ps=ps)
+  oldcex <- par ("cex")
+  par (cex=cex)
   if (is.null(col)) kol <- rainbow (xmax)
   else if (col=="gray" | col=="grey") kol <- 
-    gray (seq(0.8,0.2,length=xmax))
+    gray (seq (0.8, 0.2, length=xmax))
   else kol <- col
   xmean <- rep (0, nmerg)
   i <- 1
@@ -100,10 +101,10 @@ draw.clust <- function (cluster, ps=par("ps"), size=10,
     else {
       x1 <- cord[-a]
       y1a <- y2 - ytail
-      px <- c(x1-xbh,x1+xbh,x1+xbh,x1-xbh,x1-xbh)
-      py <- c(y1a-ybx,y1a-ybx,y1a,y1a,y1a-ybx)
-      polygon(px,py,col=kol[x1],border=0)
-      text.default (x1,y1a-(ybx/2),as.character(-a))
+      px <- c(x1-xbh, x1+xbh, x1+xbh, x1-xbh, x1-xbh)
+      py <- c(y1a-ybx, y1a-ybx, y1a, y1a, y1a-ybx)
+      polygon (px, py, col=kol[x1], border=0)
+      text.default (x1, y1a-(ybx/2), as.character(-a))
       }
     if (b > 0) {
       x2 <- xmean[b]
@@ -112,27 +113,29 @@ draw.clust <- function (cluster, ps=par("ps"), size=10,
     else {
       x2 <- cord[-b]
       y1b <- y2 - ytail
-      px <- c(x2-xbh,x2+xbh,x2+xbh,x2-xbh,x2-xbh)
-      py <- c(y1b-ybx,y1b-ybx,y1b,y1b,y1b-ybx)
-      polygon(px,py,col=kol[x2],border=0)
-      text.default (x2,y1b-(ybx/2),as.character(-b))
+      px <- c(x2-xbh, x2+xbh, x2+xbh, x2-xbh, x2-xbh)
+      py <- c(y1b-ybx, y1b-ybx, y1b, y1b, y1b-ybx)
+      polygon (px, py, col=kol[x2], border=0)
+      text.default (x2, y1b-(ybx/2), as.character(-b))
       }
     lines (c(x1,x2),c(y2,y2))
     lines (c(x1,x1),c(y1a,y2))
     lines (c(x2,x2),c(y1b,y2))
     }
-  par (ps=oldps)
+  par (cex=oldcex)
   invisible (kol)
 }
 
 ############################################################
 
-draw.tree <- function (tree, ps=par("ps"), size=10, col=NULL, 
-  nodeinfo=FALSE, units="units", cases="obs", digits=0 )
+draw.tree <- function (tree, cex=par("cex"), size=2, 
+  col=NULL, nodeinfo=FALSE, units="units", cases="obs", 
+  digits=0 )
   # tree is object of class tree
-  # ps is par parameter, pointsize of text
-  # if size=0, draw terminal symbol at leaves else a
-  #   colored square with sides of length size in mm
+  # cex is par parameter, size of text
+  # if size=0, draw terminal symbol at leaves 
+  #   else a colored square with sides of length size
+  #   in cex units
   # if col is NULL, use rainbow(),
   #   else if col="gray" or "grey", use gray()
   # if nodeinfo=TRUE, add a line at each node with mean value
@@ -145,6 +148,7 @@ draw.tree <- function (tree, ps=par("ps"), size=10, col=NULL,
 {
   rtree <- length (attr (tree, "ylevels")) == 0
   tframe <- tree$frame
+  rptree <- length (tframe$complexity) > 0
   node <- as.numeric(row.names(tframe))
   depth <- tree.depth(node)
   maxdepth <- max(depth)
@@ -174,12 +178,23 @@ draw.tree <- function (tree, ps=par("ps"), size=10, col=NULL,
   else {
     crate <- rep (0, nnodes)
     trate <- 0
-    for (i in 1:nnodes) {
-      yval <- tframe$yval[i]
-      string <- paste('tframe$yprob[,"',as.character(yval),'"]',
-          sep="")
-      crate[i] <- eval(parse(text=string))[i]
-      if (leaves[i]) trate <- trate + tframe$n[i] * crate[i]
+    if (! rptree) {
+      for (i in 1:nnodes) {
+        yval <- tframe$yval[i]
+        string <- paste('tframe$yprob[,"',
+          as.character(yval), '"]', sep="")
+        crate[i] <- eval(parse(text=string))[i]
+        if (leaves[i]) trate <- trate + tframe$n[i] * crate[i]
+        }
+      }
+    else {
+      for (i in 1:nnodes) {
+        yval <- tframe$yval[i]
+        nlv <- floor (ncol (tframe$yval2) / 2)
+        index <- rev (order (tframe$yval2[i, 2:(nlv+1)]))[1]
+        crate[i] <- tframe$yval2[i, (nlv + 1 + index)]
+        if (leaves[i]) trate <- trate + tframe$n[i] * crate[i]
+        }
       }
     crate <- round (crate,3)*100
     trate <- round (trate/tframe$n[1],3)*100
@@ -188,8 +203,8 @@ draw.tree <- function (tree, ps=par("ps"), size=10, col=NULL,
   else if (col=="gray" | col=="grey") kol <- 
     gray (seq(0.8,0.2,length=nleaves))
   else kol <- col
-  oldps <- par ("ps")
-  par (ps=ps)
+  oldcex <- par ("cex")
+  par (cex=cex)
   xmax <- max(x)
   xmin <- min(x)
   ymax <- max(y)
@@ -197,10 +212,10 @@ draw.tree <- function (tree, ps=par("ps"), size=10, col=NULL,
   pinx <- par ("pin")[1]
   piny <- par ("pin")[2]
   xscale <- (xmax - xmin)/pinx
-  box <- size / 25.4
+  chr <- par("cin")[2] * cex
+  box <- size * chr
   if (box == 0) xbh <- xscale * 0.2
   else xbh <- xscale * box/2
-  chr <- par("csi")
   tail <- box + chr
   yscale <- (ymax - ymin)/(piny - tail)
   ytail <- yscale * tail
@@ -215,8 +230,14 @@ draw.tree <- function (tree, ps=par("ps"), size=10, col=NULL,
   y1 <- ymin - yf
   y2 <- ymax + yf
   plot (c(x1,x2),c(y1,y2),type="n",axes=FALSE,xlab="",ylab="")
-  string <- paste (as.character(tframe$var[1]), "<>", 
-    substring(as.character(tframe$splits[1,1]),2))
+  if (rptree) {
+    v <- tframe$var[1]
+    n <- tframe$n[1]
+    sp <- tree$splits
+    val <- sp[(rownames(sp) == v) & (sp[,"count"] == n), "index"]
+    }
+  else val <- substring(as.character(tframe$splits[1,1]),2)
+  string <- paste (as.character(tframe$var[1]), "<>", val)
   text.default (x[1], y[1], string)
   if (nodeinfo) {
     n <- tframe$n[1]
@@ -227,14 +248,13 @@ draw.tree <- function (tree, ps=par("ps"), size=10, col=NULL,
         r,"%",sep="")
       }
     else {
-      z <- tframe$yval[1]
+      z <- attr (tree, "ylevels")[tframe$yval[1]]
       r <- crate[1]
       string <- paste (z,"; ",n," ",cases,"; ",r,"%",
         sep="")
       }
     text.default (x[1], y[1]-ychr, string)
     }
-
   for (i in 2:nnodes) {
     ytop <- ychr * (as.integer(nodeinfo)+1)
     if (y[i] < y[i-1])
@@ -243,8 +263,15 @@ draw.tree <- function (tree, ps=par("ps"), size=10, col=NULL,
       lines(c(x[parent[i]], x[i]), 
         c(y[parent[i]]-ytop, y[i]+ychr))
     if(! leaves[i]) {
-      string <- paste (as.character(tframe$var[i]), "<>", 
-        substring(as.character(tframe$splits[i,1]),2))
+      if (rptree) {
+        v <- tframe$var[i]
+        n <- tframe$n[i]
+        sp <- tree$splits
+        val <- sp[(rownames(sp) == v) & (sp[,"count"] == n), 
+          "index"]
+        }
+      else val <- substring(as.character(tframe$splits[i,1]),2)
+      string <- paste (as.character(tframe$var[i]), "<>", val)
       text.default (x[i], y[i], string)
       if (nodeinfo) {
         n <- tframe$n[i]
@@ -255,7 +282,7 @@ draw.tree <- function (tree, ps=par("ps"), size=10, col=NULL,
             r,"%",sep="")
           }
         else {
-          z <- tframe$yval[i]
+          z <- attr (tree, "ylevels")[tframe$yval[i]]
           r <- crate[i]
           string <- paste (z,"; ",n," ",cases,"; ",r,"%",
             sep="")
@@ -281,22 +308,25 @@ draw.tree <- function (tree, ps=par("ps"), size=10, col=NULL,
         z <- round(tframe$yval[i], digits)
         text.default(x[i], y[i]-ybx, paste(z,units,sep=" "))
         }
-      else text.default(x[i], y[i]-ybx, tframe$yval[i])
+      else {
+        z <- attr (tree, "ylevels")[tframe$yval[i]]
+        text.default(x[i], y[i]-ybx, z)
+        }
       n <- tframe$n[i]
       text.default(x[i], y[i]-ybx-ychr, paste(n,cases,sep=" "))
       if (box != 0)
-        text.default (x[i], y[i]-ychr/4, as.character(x[i]))
+        text.default (x[i], y[i], as.character(x[i]))
       }
     }
   if (nodeinfo) {
     if (rtree) string <- paste("Total deviance explained =",
       sum(pcor),"%")
     else string <- paste("Total classified correct =",trate,"%")
-    par (ps=1.2*ps)
+    par (cex=1.2*cex)
     if (box == 0) text.default (mean(x),ymin-3*ychr,string)
     else text.default (mean(x),ymin-ybx,string)
     }
-  par (ps=oldps)
+  par (cex=oldcex)
   invisible (kol)
 }
 
@@ -378,7 +408,7 @@ group.tree <- function (tree)
 
 ############################################################
 
-map.groups <- function (pts, group, pch=19, size=5, col=NULL,
+map.groups <- function (pts, group, pch=19, size=2, col=NULL,
   border=NULL)
   # pts must have components "x" and "y";
   # group is vector of length of number of cases (either
@@ -388,7 +418,7 @@ map.groups <- function (pts, group, pch=19, size=5, col=NULL,
   # if nrow(pts) != length(group) then map with polygon, 
   #   else if pch < 100 map with points, 
   #   else map with ngon (..., n=pch-100)
-  # size is in mm, only for point symbol
+  # size is in cex units, only for point symbol
   # if col is NULL, use rainbow(),
   #   else if col="gray" or "grey", use gray()
   # if border is NULL, use fill colors (col),
@@ -414,7 +444,7 @@ map.groups <- function (pts, group, pch=19, size=5, col=NULL,
   ncol <- length (unique (group))
   if (is.null(col)) fkol <- rainbow (ncol)
   else if (col=="gray" | col=="grey") fkol <- 
-    gray (seq(0.8, 0.2, length=ncol))
+    gray (seq (0.8, 0.2, length=ncol))
   else fkol <- rep (col, ncol)
   if (is.null(border)) bkol <- fkol
   else bkol <- rep (border, ncol)
@@ -425,27 +455,28 @@ map.groups <- function (pts, group, pch=19, size=5, col=NULL,
       col=fkol[group[j]], border=bkol[group[j]])
     }
   else if (pch < 100 | mode(pch) == "character") points (pts$x, 
-    pts$y, col=fkol[group], pch=pch, cex=1.5*size/25.4/par("cin")[1])
-  else apply (data.frame (x=pts$x, y=pts$y, d=size, c=I(fkol[group])),
+    pts$y, col=fkol[group], pch=pch, cex=size*1.5)
+  else apply (data.frame (x=pts$x, y=pts$y, 
+    d=size*25.4*par("cex")*par("cin")[1], c=I(fkol[group])),
     1, ngon, n=pch-100, type=1)
   invisible (fkol)
 }
 
 ############################################################
 
-map.key <- function (x, y, lables=NULL, new=FALSE, size=5,
-  ps=par("ps"), pch=19, head="", sep=1, col=NULL)
+map.key <- function (x, y, lables=NULL, new=FALSE, size=2,
+  cex=par("cex"), pch=19, head="", sep=0.2, col=NULL)
   # x,y are lower left coordinates of key 
   #   in proportional units (0-1)
   # lables is vector of labels for classes, or if NULL,
   #   then integers 1:length(col), or "1"
   # if new=TRUE, call plot
-  # size is (diameter) of polygon symbol in mm
-  # ps is par parameter, pointsize of text
+  # size is (diameter) of polygon symbol in cex units
+  # cex is par parameter, size of text
   # if pch < 100, use points for symbol, 
   #   else ngon (..., n=pch-100)
   # head is text heading for key
-  # sep is separation in mm between adjacent symbols
+  # sep is separation in cex units between adjacent symbols
   #   if sep=0 assume continuous scale and use gon=4
   #   and put lables at breaks between squares
   # if col is NULL, use rainbow(),
@@ -463,8 +494,8 @@ map.key <- function (x, y, lables=NULL, new=FALSE, size=5,
   else kol <- col
   if (new)
     plot(c(0,1),c(0,1),type="n",axes=FALSE,xlab="",ylab="")
-  oldps <- par("ps")
-  par (ps=ps)
+  oldcex <- par("cex")
+  par (cex=cex)
   oldadj <- par ("adj")
   par (adj=0)
   u <- par ("usr")
@@ -472,9 +503,9 @@ map.key <- function (x, y, lables=NULL, new=FALSE, size=5,
   ey <- par ("pin")[2]
   uxr <- u[2] - u[1]
   uyr <- u[4] - u[3]
-  halfx <- size/50.8
+  halfx <- (size * cex * par("cin")[1]) / 2
   xstep <- halfx + 0.05
-  ystep <- (size + sep/2.0)/25.4
+  ystep <- (size + sep) * cex * par("cin")[1]
   px <- x * uxr + u[1]
   py <- y * uyr + u[3]
   hx <- halfx * uxr / ex
@@ -485,7 +516,8 @@ map.key <- function (x, y, lables=NULL, new=FALSE, size=5,
   if (sep == 0) {
     for (i in 1:nsym) {
       qy <- qy + dy
-      ngon (c(qx, qy, size=size, col=kol[i]), n=4, type=3)
+      ngon (c(qx, qy, size=size*25.4*cex*par("cin")[1], col=kol[i]), 
+        n=4, type=3)
       text (qx+dx, qy - dy/2, lables[i]) }
     text (qx+dx, qy + dy/2, lables[nsym+1])
     }
@@ -493,8 +525,9 @@ map.key <- function (x, y, lables=NULL, new=FALSE, size=5,
     for (i in 1:nsym) {
       qy <- qy + dy
       if (pch < 100  | mode(pch) == "character") points (qx,
-        qy, col=kol[i], pch=pch, cex=1.5*size/25.4/par("cin")[1])
-      else ngon (c(qx, qy, size=size, col=kol[i]), n=pch-100, type=1)
+        qy, col=kol[i], pch=pch, cex=size*1.5)
+      else ngon (c(qx, qy, size=size*25.4*cex*par("cin")[1], col=kol[i]), 
+        n=pch-100, type=1)
       text (qx+dx, qy, lables[i]) }
   if (length(head)>0)
     {
@@ -503,7 +536,7 @@ map.key <- function (x, y, lables=NULL, new=FALSE, size=5,
     if (sep == 0) qy <- qy + 0.5 * dy
     text (qx-hx, qy, head)
     }
-  par (ps=oldps, adj=oldadj)
+  par (cex=oldcex, adj=oldadj)
   invisible (kol)
 }
 
@@ -524,10 +557,10 @@ ngon <- function (xydc, n=4, angle=0, type=1)
       # n = 4, s = 1 / sqrt(2)           = 0.7071068
       # n = 5, s = (1 + cos(.2*pi)) / 2  = 0.9045085
       # n = 6, s = sqrt(3) / 2           = 0.8660254
-  u <- par("usr")
-  p <- par("pin")
-  d <- as.numeric(xydc[3])
-  inch <- d/25.4
+  u <- par ("usr")
+  p <- par ("pin")
+  d <- as.numeric (xydc[3])
+  inch <- d / 25.4
   s <- 1
   switch (n, stop ("ngon: n=1"), 
              stop ("ngon: n=2"),
@@ -560,10 +593,10 @@ ngon <- function (xydc, n=4, angle=0, type=1)
     x[i] <- xl*costh - yl*sinth
     y[i] <- xl*sinth + yl*costh
     }
-  x <- x + as.numeric(xydc[1])
-  y <- y*ys + as.numeric(xydc[2])
-  if (type %% 2) polygon (x,y,col=xydc[4],border=0)
-  if (type %/% 2) lines (x,y,col=xydc[4])
+  x <- x + as.numeric (xydc[1])
+  y <- y*ys + as.numeric (xydc[2])
+  if (type %% 2) polygon (x, y, col=xydc[4], border=0)
+  if (type %/% 2) lines (x, y, col=xydc[4])
   invisible ()
 }
 
@@ -646,8 +679,10 @@ prune.Rpart <- function (tree, cp=NULL, best=NULL)
   ff <- tree$frame
   id <- as.integer(row.names(ff))
   if (is.null (cp)) {
-    cp <- ff$complexity
-    cp <- cp[rev (order (cp))[best]]
+    m <- tree$cptable[, "nsplit"]
+    m <- max (m[m < best])
+    m <- match (m, tree$cptable[, "nsplit"])
+    cp <- tree$cptable[m, "CP"]
     }
   toss <- id[ff$complexity <= cp & ff$var != "<leaf>"]
   if (length(toss) == 0) 
